# sage_setup: distribution = sagemath-repl
"""
This file (./sol/nonlinear_doctest.sage) was *autogenerated* from ./sol/nonlinear.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./sol/nonlinear_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./sol/nonlinear.tex, line 17::

  sage: def intervalgen(f, phi, s, t):
  ....:     assert (f(s) * f(t) < 0), \
  ....:            'Wrong arguments: f(%s) * f(%s) >= 0)'%(s, t)
  ....:     yield s
  ....:     yield t
  ....:     while 1:
  ....:         u = phi(s, t)
  ....:         yield u
  ....:         fu = f(u)
  ....:         if fu == 0:
  ....:             return
  ....:         if fu * f(s) < 0:
  ....:             t = u
  ....:         else:
  ....:             s = u

Sage example in ./sol/nonlinear.tex, line 40::

  sage: f(x) = 4 * x - 1
  sage: a, b = 0, 1
  sage: phi(s, t) = (s + t) / 2
  sage: list(intervalgen(f, phi, a, b))
  [0, 1, 1/2, 1/4]

Sage example in ./sol/nonlinear.tex, line 49::

  sage: from types import GeneratorType, FunctionType
  sage: def checklength(u, v, w, prec):
  ....:     return abs(v - u) < 2 * prec
  sage: def iterate(series, check=checklength, prec=10^-5, maxit=100):
  ....:     assert isinstance(series, GeneratorType)
  ....:     assert isinstance(check, FunctionType)
  ....:     niter = 2
  ....:     v, w = next(series), next(series)
  ....:     while (niter <= maxit):
  ....:         niter += 1
  ....:         u, v, w = v, w, next(series)
  ....:         if check(u, v, w, prec):
  ....:             print('After {0} iterations: {1}'.format(niter, w))
  ....:             return
  ....:     print('Failed after {0} iterations'.format(maxit))

Sage example in ./sol/nonlinear.tex, line 76::

  sage: f(x) = 4 * sin(x) - exp(x) / 2 + 1
  sage: a, b = RR(-pi), RR(pi)
  sage: def phi(s, t): return RR.random_element(s, t)
  sage: random = intervalgen(f, phi, a, b)
  sage: iterate(random, maxit=10000) # random
  After 19 iterations: 2.15848379485564

Sage example in ./sol/nonlinear.tex, line 93::

  sage: basering.<x> = PolynomialRing(SR, 'x')
  sage: p = x^2 + x
  sage: p.roots(multiplicities=False)
  [-1, 0]

Sage example in ./sol/nonlinear.tex, line 101::

  sage: from collections import deque
  sage: basering = PolynomialRing(SR, 'x')
  sage: q, method = None, None
  sage: def quadraticgen(f, r, s):
  ....:     global q, method
  ....:     t = r - f(r) / f.derivative()(r)
  ....:     method = 'newton'
  ....:     yield t
  ....:     pts = deque([(p, f(p)) for p in (r, s, t)], maxlen=3)
  ....:     while True:
  ....:         q = basering.lagrange_polynomial(pts)
  ....:         roots = [r for r in q.roots(multiplicities=False) \
  ....:                          if CC(r).is_real()]
  ....:         approx = None
  ....:         for root in roots:
  ....:             if (root - pts[2][0]) * (root - pts[1][0]) < 0:
  ....:                 approx = root
  ....:                 break
  ....:             elif (root - pts[0][0]) * (root - pts[1][0]) < 0:
  ....:                 pts.pop()
  ....:                 approx = root
  ....:                 break
  ....:         if approx:
  ....:             method = 'quadratic'
  ....:         else:
  ....:             method = 'dichotomy'
  ....:             approx = (pts[1][0] + pts[2][0]) / 2
  ....:         pts.append((approx, f(approx)))
  ....:         yield pts[2][0]

Sage example in ./sol/nonlinear.tex, line 141::

  sage: basering = PolynomialRing(SR, 'x')
  sage: a, b = pi/2, pi
  sage: f(x) = 4 * sin(x) - exp(x) / 2 + 1
  sage: generator = quadraticgen(f, a, b)
  sage: next(generator)
  1/2*pi - (e^(1/2*pi) - 10)*e^(-1/2*pi)
"""



# sage_setup: distribution = sagemath-repl
# sage.doctest: needs sage.rings.real_interval_field
"""
Helpers for tolerance checking in doctests
"""

# ****************************************************************************
#       Copyright (C) 2012-2018 David Roe <roed.math@gmail.com>
#                     2012      Robert Bradshaw <robertwb@gmail.com>
#                     2012      William Stein <wstein@gmail.com>
#                     2013      R. Andrew Ohana
#                     2013      Volker Braun
#                     2013-2018 Jeroen Demeyer <jdemeyer@cage.ugent.be>
#                     2016-2021 Frédéric Chapoton
#                     2017-2018 Erik M. Bray
#                     2020      Marc Mezzarobba
#                     2020-2023 Matthias Koeppe
#                     2022      John H. Palmieri
#                     2022      Sébastien Labbé
#                     2023      Kwankyu Lee
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.doctest.marked_output import MarkedOutput


_RIFtol = None


def RIFtol(*args):
    """
    Create an element of the real interval field used for doctest tolerances.

    It allows large numbers like 1e1000, it parses strings with spaces
    like ``RIF(" - 1 ")`` out of the box and it carries a lot of
    precision. The latter is useful for testing libraries using
    arbitrary precision but not guaranteed rounding such as PARI. We use
    1044 bits of precision, which should be good to deal with tolerances
    on numbers computed with 1024 bits of precision.

    The interval approach also means that we do not need to worry about
    rounding errors and it is also very natural to see a number with
    tolerance as an interval.

    EXAMPLES::

        sage: from sage.doctest.parsing import RIFtol
        sage: RIFtol(-1, 1)
        0.?
        sage: RIFtol(" - 1 ")
        -1
        sage: RIFtol("1e1000")
        1.00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000?e1000
    """
    global _RIFtol
    if _RIFtol is None:
        try:
            # We need to import from sage.all to avoid circular imports.
            from sage.rings.real_mpfi import RealIntervalField
        except ImportError:
            from warnings import warn
            warn("RealIntervalField not available, ignoring all tolerance specifications in doctests")

            def fake_RIFtol(*args):
                return 0
            _RIFtol = fake_RIFtol
        else:
            _RIFtol = RealIntervalField(1044)
    return _RIFtol(*args)


def add_tolerance(wantval, want: MarkedOutput):
    """
    Enlarge the real interval element ``wantval`` according to
    the tolerance options in ``want``.

    INPUT:

    - ``wantval`` -- a real interval element
    - ``want`` -- a :class:`MarkedOutput` describing the tolerance

    OUTPUT: an interval element containing ``wantval``

    EXAMPLES::

        sage: from sage.doctest.parsing import MarkedOutput, SageOutputChecker
        sage: from sage.doctest.rif_tol import add_tolerance
        sage: want_tol = MarkedOutput().update(tol=0.0001)
        sage: want_abs = MarkedOutput().update(abs_tol=0.0001)
        sage: want_rel = MarkedOutput().update(rel_tol=0.0001)
        sage: add_tolerance(RIF(pi.n(64)), want_tol).endpoints()                 # needs sage.symbolic
        (3.14127849432443, 3.14190681285516)
        sage: add_tolerance(RIF(pi.n(64)), want_abs).endpoints()                 # needs sage.symbolic
        (3.14149265358979, 3.14169265358980)
        sage: add_tolerance(RIF(pi.n(64)), want_rel).endpoints()                 # needs sage.symbolic
        (3.14127849432443, 3.14190681285516)
        sage: add_tolerance(RIF(1e1000), want_tol)
        1.000?e1000
        sage: add_tolerance(RIF(1e1000), want_abs)
        1.000000000000000?e1000
        sage: add_tolerance(RIF(1e1000), want_rel)
        1.000?e1000
        sage: add_tolerance(0, want_tol)
        0.000?
        sage: add_tolerance(0, want_abs)
        0.000?
        sage: add_tolerance(0, want_rel)
        0
    """
    if want.tol:
        if wantval == 0:
            return RIFtol(want.tol) * RIFtol(-1, 1)
        else:
            return wantval * (1 + RIFtol(want.tol) * RIFtol(-1, 1))
    elif want.abs_tol:
        return wantval + RIFtol(want.abs_tol) * RIFtol(-1, 1)
    elif want.rel_tol:
        return wantval * (1 + RIFtol(want.rel_tol) * RIFtol(-1, 1))
    else:
        return wantval



# sage_setup: distribution = sagemath-repl
##          Sage Doctest File         ##
#**************************************#
#*    Generated from PreTeXt source   *#
#*    on 2017-08-24T11:43:34-07:00    *#
#*                                    *#
#*   http://mathbook.pugetsound.edu   *#
#*                                    *#
#**************************************#
##
"""
Please contact Rob Beezer (beezer@ups.edu) with
any test failures here that need to be changed
as a result of changes accepted into Sage.  You
may edit/change this file in any sensible way, so
that development work may procede.  Your changes
may later be replaced by the authors of "Abstract
Algebra: Theory and Applications" when the text is
updated, and a replacement of this file is proposed
for review.
"""
##
## To execute doctests in these files, run
##   $ $SAGE_ROOT/sage -t <directory-of-these-files>
## or
##   $ $SAGE_ROOT/sage -t <a-single-file>
##
## Replace -t by "-tp n" for parallel testing,
##   "-tp 0" will use a sensible number of threads
##
## See: http://www.sagemath.org/doc/developer/doctesting.html
##   or run  $ $SAGE_ROOT/sage --advanced  for brief help
##
## Generated at 2017-08-24T11:43:34-07:00
## From "Abstract Algebra"
## At commit 26d3cac0b4047f4b8d6f737542be455606e2c4b4
##
## Section 2.6 Sage
##
r"""
~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: r = 14 % 3
    sage: r
    2

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: q = (14 - r)/3
    sage: q
    4

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a = 14
    sage: b = 3
    sage: a.quo_rem(b)
    (4, 2)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: (20 % 5) == 0
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: (17 % 4) == 0
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: c = 5
    sage: c.divides(20)
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: d = 4
    sage: d.divides(17)
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: gcd(2776, 2452)
    4

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a = 31049
    sage: b = 2105
    sage: gcd(a, b) == 1
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a = 3563
    sage: b = 2947
    sage: gcd(a, b) == 1
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: xgcd(633,331)
    (1, -137, 262)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a = 633
    sage: b = 331
    sage: extended = xgcd(a, b)
    sage: g = extended[0]
    sage: r = extended[1]
    sage: s = extended[2]
    sage: g == r*a + s*b
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a = 117371
    sage: a.is_prime()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: b = 14547073
    sage: b.is_prime()
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: b == 1597 * 9109
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a = random_prime(10^21, proof=True)
    sage: a   # random
    424729101793542195193

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a.is_prime()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: prime_range(500, 550)
    [503, 509, 521, 523, 541, 547]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a = 2600
    sage: a.factor()
    2^3 * 5^2 * 13

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a = 2600
    sage: factored = a.factor()
    sage: first_term = factored[0]
    sage: first_term
    (2, 3)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: second_term = factored[1]
    sage: second_term
    (5, 2)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: third_term = factored[2]
    sage: third_term
    (13, 1)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: first_prime = first_term[0]
    sage: first_prime
    2

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: first_exponent = first_term[1]
    sage: first_exponent
    3

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: list(factored)
    [(2, 3), (5, 2), (13, 1)]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: len(factored)
    3
"""



# sage_setup: distribution = sagemath-repl
##          Sage Doctest File         ##
#**************************************#
#*    Generated from PreTeXt source   *#
#*    on 2017-08-24T11:43:34-07:00    *#
#*                                    *#
#*   http://mathbook.pugetsound.edu   *#
#*                                    *#
#**************************************#
##
"""
Please contact Rob Beezer (beezer@ups.edu) with
any test failures here that need to be changed
as a result of changes accepted into Sage.  You
may edit/change this file in any sensible way, so
that development work may procede.  Your changes
may later be replaced by the authors of "Abstract
Algebra: Theory and Applications" when the text is
updated, and a replacement of this file is proposed
for review.
"""
##
## To execute doctests in these files, run
##   $ $SAGE_ROOT/sage -t <directory-of-these-files>
## or
##   $ $SAGE_ROOT/sage -t <a-single-file>
##
## Replace -t by "-tp n" for parallel testing,
##   "-tp 0" will use a sensible number of threads
##
## See: http://www.sagemath.org/doc/developer/doctesting.html
##   or run  $ $SAGE_ROOT/sage --advanced  for brief help
##
## Generated at 2017-08-24T11:43:34-07:00
## From "Abstract Algebra"
## At commit 26d3cac0b4047f4b8d6f737542be455606e2c4b4
##
## Section 9.4 Sage
##
r"""
~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: m = 12
    sage: n = 7
    sage: gcd(m, n)
    1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = CyclicPermutationGroup(m)
    sage: H = CyclicPermutationGroup(n)
    sage: dp = direct_product_permgroups([G, H])
    sage: K = CyclicPermutationGroup(m*n)
    sage: K.is_isomorphic(dp)
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: m = 15
    sage: n = 21
    sage: gcd(m, n)
    3

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = CyclicPermutationGroup(m)
    sage: H = CyclicPermutationGroup(n)
    sage: dp = direct_product_permgroups([G, H])
    sage: K = CyclicPermutationGroup(m*n)
    sage: K.is_isomorphic(dp)
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: m = 6
    sage: n = 5
    sage: r = 7
    sage: G = CyclicPermutationGroup(m)
    sage: H = CyclicPermutationGroup(n)
    sage: L = CyclicPermutationGroup(r)
    sage: dp = direct_product_permgroups([G, H, L])
    sage: K = CyclicPermutationGroup(m*n*r)
    sage: K.is_isomorphic(dp)
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: [CyclicPermutationGroup(p) for p in [1, 2, 3, 5, 7, 11, 13]]
    [Cyclic group of order 1 as a permutation group,
     Cyclic group of order 2 as a permutation group,
     Cyclic group of order 3 as a permutation group,
     Cyclic group of order 5 as a permutation group,
     Cyclic group of order 7 as a permutation group,
     Cyclic group of order 11 as a permutation group,
     Cyclic group of order 13 as a permutation group]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = CyclicPermutationGroup(4)
    sage: H = KleinFourGroup()
    sage: T1 = CyclicPermutationGroup(2)
    sage: T2 = CyclicPermutationGroup(2)
    sage: K = direct_product_permgroups([T1, T2])
    sage: G.is_isomorphic(H)
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: H.is_isomorphic(K)
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = CyclicPermutationGroup(6)
    sage: H = SymmetricGroup(3)
    sage: G.is_isomorphic(H)
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = DihedralGroup(3)
    sage: H = SymmetricGroup(3)
    sage: G.is_isomorphic(H)
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Z36 = Integers(36)
    sage: U = [x for x in Z36 if gcd(ZZ(x), 36) == 1]
    sage: U
    [1, 5, 7, 11, 13, 17, 19, 23, 25, 29, 31, 35]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: [x.multiplicative_order() for x in U]
    [1, 6, 6, 6, 3, 2, 2, 6, 3, 6, 6, 2]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a = U[1]
    sage: A = [a^i for i in srange(6)]
    sage: A
    [1, 5, 25, 17, 13, 29]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: b = U[11]
    sage: B = [b^i for i in srange(2)]
    sage: B
    [1, 35]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: [x for x in A if x in B]
    [1]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: all(x*y == y*x for x in A for y in B)
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: T = sorted(x*y for x in A for y in B)
    sage: T
    [1, 5, 7, 11, 13, 17, 19, 23, 25, 29, 31, 35]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: T == U
    True
"""



# sage_setup: distribution = sagemath-repl
##          Sage Doctest File         ##
#**************************************#
#*    Generated from PreTeXt source   *#
#*    on 2017-08-24T11:43:34-07:00    *#
#*                                    *#
#*   http://mathbook.pugetsound.edu   *#
#*                                    *#
#**************************************#
##
"""
Please contact Rob Beezer (beezer@ups.edu) with
any test failures here that need to be changed
as a result of changes accepted into Sage.  You
may edit/change this file in any sensible way, so
that development work may procede.  Your changes
may later be replaced by the authors of "Abstract
Algebra: Theory and Applications" when the text is
updated, and a replacement of this file is proposed
for review.
"""
##
## To execute doctests in these files, run
##   $ $SAGE_ROOT/sage -t <directory-of-these-files>
## or
##   $ $SAGE_ROOT/sage -t <a-single-file>
##
## Replace -t by "-tp n" for parallel testing,
##   "-tp 0" will use a sensible number of threads
##
## See: http://www.sagemath.org/doc/developer/doctesting.html
##   or run  $ $SAGE_ROOT/sage --advanced  for brief help
##
## Generated at 2017-08-24T11:43:34-07:00
## From "Abstract Algebra"
## At commit 26d3cac0b4047f4b8d6f737542be455606e2c4b4
##
## Section 23.6 Sage
##
r"""
~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: x = polygen(QQ, 'x')
    sage: N.<a> = NumberField(x^4 - 2); N
    Number Field in a with defining polynomial x^4 - 2

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: L.<b> = N.galois_closure(); L
    Number Field in b with defining polynomial x^8 + 28*x^4 + 2500

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: L.degree()
    8

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: y = polygen(L, 'y')
    sage: (y^4 - 2).factor()
    (y - 1/120*b^5 -  19/60*b) *
    (y - 1/240*b^5 + 41/120*b) *
    (y + 1/240*b^5 - 41/120*b) *
    (y + 1/120*b^5 +  19/60*b)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = L.galois_group(); G
    Galois group 8T4 ([4]2) with order 8 of x^8 + 28*x^4 + 2500

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G.is_abelian()
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G.order()
    8

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G.list()
    [(), (1,2,8,7)(3,4,6,5),
    (1,3)(2,5)(4,7)(6,8), (1,4)(2,3)(5,8)(6,7),
    (1,5)(2,6)(3,7)(4,8), (1,6)(2,4)(3,8)(5,7),
    (1,7,8,2)(3,5,6,4), (1,8)(2,7)(3,6)(4,5)]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G.is_isomorphic(DihedralGroup(4))
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: x = polygen(QQ, 'x')
    sage: p = x^4 - 2
    sage: N.<a> = NumberField(p); N
    Number Field in a with defining polynomial x^4 - 2

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: y = polygen(N, 'y')
    sage: p = p.subs(x=y)
    sage: p.factor()
    (y - a) * (y + a) * (y^2 + a^2)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: M.<b> = NumberField(y^2 + a^2); M
    Number Field in b with defining polynomial y^2 + a^2 over
    its base field

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: z = polygen(M, 'z')
    sage: (z^4 - 2).factor()
    (z - b) * (z - a) * (z + a) * (z + b)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: L.<c> = M.absolute_field(); L
    Number Field in c with defining polynomial x^8 + 28*x^4 + 2500

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL, toL = L.structure()

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: roots = p.roots(ring=L, multiplicities=False); roots
    [1/120*c^5 +  19/60*c,
     1/240*c^5 - 41/120*c,
    -1/240*c^5 + 41/120*c,
    -1/120*c^5 -  19/60*c]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: [fromL(r) for r in roots]
    [b, a, -a, -b]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = End(L); G
    Automorphism group of Number Field in c with
    defining polynomial x^8 + 28*x^4 + 2500

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: [tau(1) for tau in G]
    [1, 1, 1, 1, 1, 1, 1, 1]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: Sequence([[fromL(tau(r)) for r in roots] for tau in G], cr=True)
    [[b, a, -a, -b],
     [-b, -a, a, b],
     [a, -b, b, -a],
     [b, -a, a, -b],
     [-a, -b, b, a],
     [a, b, -b, -a],
     [-b, a, -a, b],
     [-a, b, -b, a]]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: S4 = SymmetricGroup(4)
    sage: elements = [S4([1, 2, 3, 4]),
    ....:             S4([4, 3, 2, 1]),
    ....:             S4([2, 4, 1, 3]),
    ....:             S4([1, 3, 2, 4]),
    ....:             S4([3, 4, 1, 2]),
    ....:             S4([2, 1, 4, 3]),
    ....:             S4([4, 2, 3, 1]),
    ....:             S4([3, 1, 4, 2])]
    sage: elements
    [(), (1,4)(2,3), (1,2,4,3), (2,3), (1,3)(2,4),
     (1,2)(3,4), (1,4), (1,3,4,2)]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: P = S4.subgroup(elements)
    sage: P.is_isomorphic(DihedralGroup(4))
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: basis = L.power_basis(); basis
    [1, c, c^2, c^3, c^4, c^5, c^6, c^7]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: tau = G[3]
    sage: z = 4 + 5*c+ 6*c^3-7*c^6
    sage: tz = tau(4 + 5*c+ 6*c^3-7*c^6); tz
    11/250*c^7 - 98/25*c^6 + 1/12*c^5 + 779/125*c^3 +
    6006/25*c^2 - 11/6*c + 4

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: tz.vector()
    (4, -11/6, 6006/25, 779/125, 0, 1/12, -98/25, 11/250)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: tau_matrix = column_matrix([tau(be).vector() for be in basis])
    sage: tau_matrix
    [  1       0       0        0  -28       0        0          0]
    [  0  -11/30       0        0    0  779/15        0          0]
    [  0       0  -14/25        0    0       0  -858/25          0]
    [  0       0       0  779/750    0       0        0  -4031/375]
    [  0       0       0        0   -1       0        0          0]
    [  0    1/60       0        0    0   11/30        0          0]
    [  0       0   -1/50        0    0       0    14/25          0]
    [  0       0       0  11/1500    0       0        0   -779/750]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: tau_matrix*z.vector()
    (4, -11/6, 6006/25, 779/125, 0, 1/12, -98/25, 11/250)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: tau_matrix*(z.vector()) == (tau(z)).vector()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: K = (tau_matrix-identity_matrix(8)).right_kernel(); K
    Vector space of degree 8 and dimension 4 over Rational Field
    Basis matrix:
    [    1     0     0     0     0     0     0     0]
    [    0     1     0     0     0  1/38     0     0]
    [    0     0     1     0     0     0 -1/22     0]
    [    0     0     0     1     0     0     0 1/278]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL(1)
    1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL(c + (1/38)*c^5)
    60/19*b

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL(c^2 - (1/22)*c^6)
    150/11*a^2

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL(c^3 + (1/278)*c^7)
    1500/139*a^2*b

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a^2 + b^2
    0

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: sg = P.subgroups()
    sage: [H.gens() for H in sg]
    [((),),
     ((1,4)(2,3),),
     ((2,3),),
     ((1,4),),
     ((1,2)(3,4),),
     ((1,3)(2,4),),
     ((1,4)(2,3), (2,3)),
     ((1,4)(2,3), (1,2,4,3)),
     ((1,4)(2,3), (1,2)(3,4)),
     ((1,4)(2,3), (2,3), (1,2,4,3))]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: [H.order() for H in sg]
    [1, 2, 2, 2, 2, 2, 4, 4, 4, 8]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: tau = G[4]
    sage: tau_matrix = column_matrix([tau(be).vector() for be in basis])
    sage: (tau_matrix-identity_matrix(8)).right_kernel()
    Vector space of degree 8 and dimension 4 over Rational Field
    Basis matrix:
    [     1      0      0      0      0      0      0      0]
    [     0      1      0      0      0  1/158      0      0]
    [     0      0      1      0      0      0   1/78      0]
    [     0      0      0      1      0      0      0 13/614]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL(tau(1))
    1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL(tau(c+(1/158)*c^5))
    120/79*b - 120/79*a

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL(tau(c^2+(1/78)*c^6))
    -200/39*a*b

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL(tau(c^3+(13/614)*c^7))
    3000/307*a^2*b + 3000/307*a^3

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: (b-a)^2
    -2*a*b

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: (b-a)^3
    2*a^2*b + 2*a^3

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: subinfo = L.subfield((79/120)*(c+(1/158)*c^5)); subinfo
    (Number Field in c0 with defining polynomial x^4 + 8 with c0 = 1/240*c^5 + 79/120*c,
     Ring morphism:
       From: Number Field in c0 with defining polynomial x^4 + 8 with c0 = 1/240*c^5 + 79/120*c
       To:   Number Field in c with defining polynomial x^8 + 28*x^4 + 2500
       Defn: c0 |--> 1/240*c^5 + 79/120*c)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: V = QQ^8
    sage: for tau in [G[0], G[1], G[3], G[6]]:
    ....:   tau_matrix = column_matrix([tau(be).vector() for be in basis])
    ....:   K = (tau_matrix-identity_matrix(8)).right_kernel()
    ....:   V = V.intersection(K)
    sage: V
    Vector space of degree 8 and dimension 2 over Rational Field
    Basis matrix:
    [    1     0     0     0     0     0     0     0]
    [    0     0     1     0     0     0 -1/22     0]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL(tau(c^2 - (1/22)*c^6))
    150/11*a^2

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: F, mapping = L.subfield((11/150)*(c^2 - (1/22)*c^6))
    sage: F
    Number Field in c0 with defining polynomial x^2 - 2 with c0 = -1/300*c^6 + 11/150*c^2

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: V = QQ^8
    sage: for tau in [G[0], G[1], G[2], G[7]]:
    ....:   tau_matrix = column_matrix([tau(be).vector() for be in basis])
    ....:   K = (tau_matrix-identity_matrix(8)).right_kernel()
    ....:   V = V.intersection(K)
    sage: V
    Vector space of degree 8 and dimension 2 over Rational Field
    Basis matrix:
    [1 0 0 0 0 0 0 0]
    [0 0 0 0 1 0 0 0]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL(tau(c^4))
    -24*a^3*b - 14

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: F, mapping = L.subfield((c^4+14)/-48)
    sage: F
    Number Field in c0 with defining polynomial x^2 + 1 with c0 = -1/48*c^4 - 7/24

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: L.subfields()
    [(Number Field in c0 with defining polynomial x,
      Ring morphism:
        From: Number Field in c0 with defining polynomial x
        To:   Number Field in c with defining polynomial x^8 + 28*x^4 + 2500
        Defn: 0 |--> 0,
      None),
     (Number Field in c1 with defining polynomial x^2 + 112*x + 40000,
      Ring morphism:
        From: Number Field in c1 with defining polynomial x^2 + 112*x + 40000
        To:   Number Field in c with defining polynomial x^8 + 28*x^4 + 2500
        Defn: c1 |--> 4*c^4,
      None),
     (Number Field in c2 with defining polynomial x^2 + 512,
      Ring morphism:
        From: Number Field in c2 with defining polynomial x^2 + 512
        To:   Number Field in c with defining polynomial x^8 + 28*x^4 + 2500
        Defn: c2 |--> 1/25*c^6 + 78/25*c^2,
      None),
     (Number Field in c3 with defining polynomial x^2 - 288,
      Ring morphism:
        From: Number Field in c3 with defining polynomial x^2 - 288
        To:   Number Field in c with defining polynomial x^8 + 28*x^4 + 2500
        Defn: c3 |--> -1/25*c^6 + 22/25*c^2,
      None),
     (Number Field in c4 with defining polynomial x^4 + 112*x^2 + 40000,
      Ring morphism:
        From: Number Field in c4 with defining polynomial x^4 + 112*x^2 + 40000
        To:   Number Field in c with defining polynomial x^8 + 28*x^4 + 2500
        Defn: c4 |--> 2*c^2,
      None),
     (Number Field in c5 with defining polynomial x^4 + 8,
      Ring morphism:
        From: Number Field in c5 with defining polynomial x^4 + 8
        To:   Number Field in c with defining polynomial x^8 + 28*x^4 + 2500
        Defn: c5 |--> -1/80*c^5 + 1/40*c,
      None),
     (Number Field in c6 with defining polynomial x^4 + 648,
      Ring morphism:
        From: Number Field in c6 with defining polynomial x^4 + 648
        To:   Number Field in c with defining polynomial x^8 + 28*x^4 + 2500
        Defn: c6 |--> 1/80*c^5 + 79/40*c,
      None),
     (Number Field in c7 with defining polynomial x^4 - 512,
      Ring morphism:
        From: Number Field in c7 with defining polynomial x^4 - 512
        To:   Number Field in c with defining polynomial x^8 + 28*x^4 + 2500
        Defn: c7 |--> -1/60*c^5 + 41/30*c,
      None),
     (Number Field in c8 with defining polynomial x^4 - 32,
      Ring morphism:
        From: Number Field in c8 with defining polynomial x^4 - 32
        To:   Number Field in c with defining polynomial x^8 + 28*x^4 + 2500
        Defn: c8 |--> 1/60*c^5 + 19/30*c,
      None),
     (Number Field in c9 with defining polynomial x^8 + 28*x^4 + 2500,
      Ring morphism:
        From: Number Field in c9 with defining polynomial x^8 + 28*x^4 + 2500
        To:   Number Field in c with defining polynomial x^8 + 28*x^4 + 2500
        Defn: c9 |--> c,
      Ring morphism:
        From: Number Field in c with defining polynomial x^8 + 28*x^4 + 2500
        To:   Number Field in c9 with defining polynomial x^8 + 28*x^4 + 2500
        Defn: c |--> c9)]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: tau = G[6]
    sage: tau_matrix = column_matrix([tau(be).vector() for be in basis])
    sage: (tau_matrix-identity_matrix(8)).right_kernel()
    Vector space of degree 8 and dimension 4 over Rational Field
    Basis matrix:
    [    1     0     0     0     0     0     0     0]
    [    0     1     0     0     0 -1/82     0     0]
    [    0     0     1     0     0     0 -1/22     0]
    [    0     0     0     1     0     0     0 11/58]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL(tau(1))
    1

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL(tau(c+(-1/82)*c^5))
    -120/41*a

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL(tau(c^2+(-1/22)*c^6))
    150/11*a^2

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: fromL(tau(c^3+(11/58)*c^7))
    3000/29*a^3

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: sg[2].is_normal(P)
    False
"""



# sage_setup: distribution = sagemath-repl
"""
This file (./premierspas_doctest.sage) was *autogenerated* from ./premierspas.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./premierspas_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./premierspas.tex, line 589::

  sage: 1+1
  2

Sage example in ./premierspas.tex, line 645::

  sage: ( 1 + 2 * (3 + 5) ) * 2
  34

Sage example in ./premierspas.tex, line 656::

  sage: 2^3
  8
  sage: 2**3
  8

Sage example in ./premierspas.tex, line 664::

  sage: 20/6
  10/3

Sage example in ./premierspas.tex, line 700::

  sage: 20.0 / 14
  1.42857142857143

Sage example in ./premierspas.tex, line 724::

  sage: numerical_approx(20/14, digits=60)
  1.42857142857142857142857142857142857142857142857142857142857

Sage example in ./premierspas.tex, line 792::

  sage: 20 // 6
  3
  sage: 20 % 6
  2

Sage example in ./premierspas.tex, line 818::

  sage: factor(2^(2^5)+1)
  641 * 6700417

Sage example in ./premierspas.tex, line 952::

  sage: sin(pi)
  0
  sage: tan(pi/3)
  sqrt(3)
  sage: arctan(1)
  1/4*pi
  sage: exp(2 * I * pi)
  1

Sage example in ./premierspas.tex, line 967::

  sage: arccos(sin(pi/3))
  arccos(1/2*sqrt(3))
  sage: sqrt(2)
  sqrt(2)
  sage: exp(I*pi/7)
  e^(1/7*I*pi)

Sage example in ./premierspas.tex, line 987::

  sage: simplify(arccos(sin(pi/3)))
  1/6*pi

Sage example in ./premierspas.tex, line 1000::

  sage: numerical_approx(6*arccos(sin(pi/3)), digits=60)
  3.14159265358979323846264338327950288419716939937510582097494
  sage: numerical_approx(sqrt(2), digits=60)
  1.41421356237309504880168872420969807856967187537694807317668

Sage example in ./premierspas.tex, line 1139::

  sage: y = 1 + 2

Sage example in ./premierspas.tex, line 1144::

  sage: y
  3
  sage: (2 + y) * y
  15

Sage example in ./premierspas.tex, line 1156::

  sage: y = 1 + 2; y
  3

Sage example in ./premierspas.tex, line 1168::

  sage: y = 3 * y + 1; y
  10
  sage: y = 3 * y + 1; y
  31
  sage: y = 3 * y + 1; y
  94

Sage example in ./premierspas.tex, line 1208::

  sage: pi = -I/2
  sage: exp(2*I*pi)
  e

Sage example in ./premierspas.tex, line 1217::

  sage: from sage.symbolic.constants import pi

Sage example in ./premierspas.tex, line 1224::

  sage: restore()

Sage example in ./premierspas.tex, line 1267::

  sage: z = SR.var('z')
  sage: 2*z + 3
  2*z + 3

Sage example in ./premierspas.tex, line 1290::

  sage: y = SR.var('z')
  sage: 2*y + 3
  2*z + 3

Sage example in ./premierspas.tex, line 1306::

  sage: c = 2 * y + 3
  sage: z = 1
  sage: 2*y + 3
  2*z + 3
  sage: c
  2*z + 3

Sage example in ./premierspas.tex, line 1318::

  sage: x = SR.var('x')
  sage: expr = sin(x); expr
  sin(x)
  sage: expr(x=1)
  sin(1)

Sage example in ./premierspas.tex, line 1332::

  sage: u = SR.var('u')
  sage: u = u+1
  sage: u = u+1
  sage: u
  u + 2

Sage example in ./premierspas.tex, line 1347::

  sage: x = SR.var('x', 100)
  sage: (x[0] + x[1])*x[99]
  (x0 + x1)*x99

Sage example in ./premierspas.tex, line 1357::

  sage: var('a, b, c, x, y')
  (a, b, c, x, y)
  sage: a * x + b * y + c
  a*x + b*y + c

Sage example in ./premierspas.tex, line 1380::

  sage: var('bla')
  bla
"""
